@@ -343,6 +343,8 @@ ExtractFromXIF <- function(fileName, extract_features = TRUE, extract_images = F
         features$`Object Number` = 0:(nrow(features)-1)
         warning(paste0("found duplicated objects when reading file: ", fileName))
       }
+      features = getFeaturesValues(features_def = features_def[sapply(features_def, FUN = function(f_def) f_def$type == "combined")],
+                                   features = features)[, features_names]
       rownames(features) = 0:(nrow(features)-1)
       class(features) <- c(class(features),"IFC_features")
       class(features_def) <- c(class(features_def),"IFC_features_def")
@@ -459,19 +461,7 @@ ExtractFromXIF <- function(fileName, extract_features = TRUE, extract_images = F
                         display_progress = display_progress,
                         title_progress = title_progress, ...)
       
-      if(extract_stats) {
-        stats = data.frame(stringsAsFactors = FALSE, check.rows = FALSE, check.names = FALSE, t(sapply(names(pops), FUN=function(p) {
-          count = sum(pops[[p]]$obj)
-          base = pops[[p]]$base
-          type = pops[[p]]$type
-          if(base=="") base = "All"
-          parent = sum(pops[[base]]$obj)
-          c("type" = type, "parent" = base, "count" = count, "perc_parent" = count/parent*100, "perc_tot" = count/obj_number*100)
-        })))
-        stats[,3] = as.numeric(stats[,3])
-        stats[,4] = as.numeric(stats[,4])
-        stats[,5] = as.numeric(stats[,5])
-      }
+      if(extract_stats) stats = get_pops_stats(pops, obj_number)
     }
   } else {
     features = data.frame()
