@@ -51,12 +51,14 @@
 #' Note also that an original file, i.e. generated by IDEAS(R) or INSPIRE(R), will never be overwritten.
 #' Otherwise, you will get an error saying that overwriting original file is not allowed.\cr
 #' @param display_progress whether to display a progress bar. Default is TRUE.
+#' @param add_tracking whether to register files' paths and objects' ids in the exported file. Default is TRUE.
 #' @param ... other arguments to be passed.
 #' @return It invisibly returns full path of exported file.
 #' @keywords internal
 mergeXIF <- function (fileName, write_to,
                       extract_features = FALSE, endianness = .Platform$endian, verbose = FALSE, verbosity = 1, 
-                      overwrite = FALSE, display_progress = TRUE, ...) {
+                      overwrite = FALSE, display_progress = TRUE, 
+                      add_tracking = TRUE, ...) {
   dots = list(...)
   # change locale
   locale_back = Sys.getlocale("LC_ALL")
@@ -75,6 +77,7 @@ mergeXIF <- function (fileName, write_to,
   if(!all(tmp)) stop(paste0(paste0("'fileName', can't find file",ifelse(sum(!tmp) > 1, "s:", ":")),"\n-", paste0(fileName[!tmp],collapse="\n-")))
   tmp = unique(getFileExt(fileName))
   if(length(tmp) != 1) stop("all files in 'fileName' should have same extension")
+  fileName = enc2native(normalizePath(fileName, winslash = "/", mustWork = FALSE))
   tryCatch({
     tmp = unique(sapply(fileName, cpp_checkTIFF))
   }, error = function(e) {
@@ -84,15 +87,13 @@ mergeXIF <- function (fileName, write_to,
   XIF_test = unique(sapply(fileName, testXIF))
   if(length(XIF_test) != 1) stop("can't deal with files in 'fileName' with different image/mask storage")
   XIF_step = as.integer(XIF_test == 1) + 1L
-  
-  fileName = normalizePath(fileName, winslash = "/", mustWork = FALSE)
   file_first = fileName[1]
   
   # no check on illumination
   # check on channels
   tmp = lapply(fileName, FUN = function(f) {
     IFD = getIFD(fileName = f, offsets = "first", verbose = verbose, verbosity = verbosity, bypass = TRUE)
-    tmp_acq = read_xml(getFullTag(IFD = IFD, which = 1, "33027"), options=c("HUGE","RECOVER","NOENT","NOBLANKS","NSCLEAN"))
+    tmp_acq = read_xml(getFullTag(IFD = IFD, which = 1, tag = "33027", raw = TRUE), options=c("HUGE","RECOVER","NOENT","NOBLANKS","NSCLEAN"))
     as.logical(as.numeric(unlist(strsplit( xml_text(xml_find_first(tmp_acq, "//Imaging//ChannelInUseIndicators_0_11")), " ", useBytes = TRUE, fixed = TRUE))))
   })
   if(!do.call(what = "all.equal.list", args = tmp)) stop("files in 'fileName' have been acquired with different channels")
@@ -101,6 +102,7 @@ mergeXIF <- function (fileName, write_to,
   extract_features = as.logical(extract_features); assert(extract_features, len = 1, alw = c(TRUE, FALSE))
   display_progress = as.logical(display_progress); assert(display_progress, len = 1, alw = c(TRUE, FALSE))
   verbose = as.logical(verbose); assert(verbose, len = 1, alw = c(TRUE, FALSE))
+  add_tracking = as.logical(add_tracking); assert(add_tracking, len = 1, alw = c(TRUE, FALSE))
   if(verbose) {
     verbosity = as.integer(verbosity); assert(verbosity, len = 1, alw = c(1,2))
     VER = ifelse(verbose & (verbosity==2), TRUE, FALSE)
@@ -194,10 +196,11 @@ mergeXIF <- function (fileName, write_to,
     
     # set additional IFD
     # 33029 or 33030 names of files that constitute the merge
+    fname2 = collapse_raw(lapply(fileName, FUN = function(x) charToRaw(normalizePath(enc2native(x), mustWork = FALSE, winslash =  "\\"))), as.raw(0x7c))
     if(f_Ext == "rif") {
-      ifd_merged = buildIFD(val = paste0(normalizePath(fileName, winslash = "\\"), collapse = "|"), typ = 2, tag = 33030, endianness = r_endian)
+      ifd_merged = buildIFD(val = fname2, typ = 2, tag = 33030, endianness = r_endian)
     } else {
-      ifd_merged = buildIFD(val = paste0(normalizePath(fileName, winslash = "\\"), collapse = "|"), typ = 2, tag = 33029, endianness = r_endian)
+      ifd_merged = buildIFD(val = fname2, typ = 2, tag = 33029, endianness = r_endian)
     }
     # 33004 now time
     ifd_time = buildIFD(val = format(Sys.time(), "%d-%m-%Y %H:%M:%S %p"), typ = 2, tag = 33004, endianness = r_endian)
@@ -207,17 +210,22 @@ mergeXIF <- function (fileName, write_to,
     ifd_obj = buildIFD(val = final_obj, typ = 4, tag = 33018, endianness = r_endian)
     # 33090 ifc pkg version
     ifd_version = buildIFD(val = paste0(unlist(packageVersion("IFC")), collapse = "."), typ = 2, tag = 33090, endianness = r_endian)
-    # 33091 names of files that constitute the merge
-    ifd_files = buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = IFD_first, which = 1, tag = "33091")),
-                                        paste0(normalizePath(fileName, winslash = "\\"), collapse = "|")),
-                                      collapse = ">"),
-                         typ = 2, tag = 33091, endianness = r_endian)
-    # 33092 checksum of each file constituting the merge
-    ifd_checksum = buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = IFD_first, which = 1, tag = "33092")),
-                                           paste0(unname(sapply(fileName, cpp_checksum)), collapse = "|")),
-                                         collapse = ">"), 
-                            typ = 2, tag = 33092, endianness = r_endian)
-    
+    if(add_tracking) {
+      # 33091 names of files that constitute the merge
+      ifd_files = buildIFD(val = collapse_raw(c(list(suppressWarnings(getFullTag(IFD = IFD_first, which = 1, tag = "33091", raw = TRUE))),
+                                                list(fname2)),
+                                              collapse = as.raw("0x3e")),
+                           typ = 2, tag = 33091, endianness = r_endian)
+      # 33092 checksum of each file constituting the merge
+      ifd_checksum = buildIFD(val = collapse_raw(c(list(suppressWarnings(getFullTag(IFD = IFD_first, which = 1, tag = "33092", raw = TRUE))),
+                                                   list(collapse_raw(lapply(fileName, FUN = function(x) charToRaw(num_to_string(cpp_checksum(x)))), as.raw(0x7c)))),
+                                                 collapse = as.raw("0x3e")),
+                              typ = 2, tag = 33092, endianness = r_endian)
+    } else {
+      ifd_files = list()
+      ifd_checksum = list()
+    }
+
     pos = writeIFD(ifd = IFD$tags[!(names(IFD$tags) %in% unwanted)],
                    r_con = toread1, w_con = towrite, pos = pos, 
                    extra = c(ifd_time, ifd_user, ifd_obj, ifd_merged, ifd_version, ifd_files, ifd_checksum),
@@ -235,6 +243,7 @@ mergeXIF <- function (fileName, write_to,
     
     # repeat same process for img / msk data for all files
     for(f in fileName) {
+      fname3 = charToRaw(f)
       IFD_first = getIFD(fileName = f, offsets = "first", trunc_bytes = 4, force_trunc = TRUE, verbose = verbose, verbosity = verbosity, bypass = TRUE)
       obj_count = XIF_step * getFullTag(IFD = IFD_first, which = 1, tag = "33018")
       IFD = IFD_first[[1]]
@@ -258,17 +267,21 @@ mergeXIF <- function (fileName, write_to,
           if(endianness != r_endian) tmp = rev(tmp)
           if(length(ifd[["33003"]])!=0) ifd[["33003"]]$raw[9:12] <- tmp
           
-          extra = c(
-            # register current object id in new tag to be able to track it
-            buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = f), which = 1, tag = "33093")),
-                                    OBJECT_ID),
-                                  collapse = ">"),
-                     typ = 2, tag = 33093, endianness = r_endian),
-            # add origin fileName to allow to track where exported objects are coming from
-            buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = f), which = 1, tag = "33094")),
-                                    f),
-                                  collapse = ">"),
-                     typ = 2, tag = 33094, endianness = r_endian))
+          if(add_tracking) {
+            extra = c(
+              # register current object id in new tag to be able to track it
+              buildIFD(val = collapse_raw(c(list(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = f), which = 1, tag = "33093", raw = TRUE))),
+                                            list(charToRaw(num_to_string(OBJECT_ID)))),
+                                          collapse = as.raw(0x3e)),
+                       typ = 2, tag = 33093, endianness = r_endian),
+              # add origin fileName to allow to track where exported objects are coming from
+              buildIFD(val = collapse_raw(c(list(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = f), which = 1, tag = "33094", raw = TRUE))),
+                                            list(fname3)),
+                                          collapse = as.raw(0x3e)),
+                       typ = 2, tag = 33094, endianness = r_endian))
+          } else {
+            extra = NULL
+          }
           
           pos = writeIFD(ifd, r_con = toread, w_con = towrite, pos = pos, extra = extra, endianness = r_endian)
         }
