@@ -61,6 +61,7 @@
 #' Note also that an original file, i.e. generated by IDEAS(R) or INSPIRE(R), will never be overwritten.
 #' Otherwise, you will get an error saying that overwriting original file is not allowed.\cr
 #' @param display_progress whether to display a progress bar. Default is TRUE.
+#' @param add_tracking whether to register files' paths and objects' ids in the exported file. Default is TRUE.
 #' @param ... other arguments to be passed.
 #' @details when 'extract_features' is set TRUE, only features stored in binary format will be extracted if found.\cr
 #' If the input 'fileName' is a merged of several files then features will be extracted from these files.\cr
@@ -69,7 +70,8 @@
 #' @keywords internal
 subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE, 
                        extract_features = FALSE, endianness = .Platform$endian, verbose = FALSE, verbosity = 1, 
-                       overwrite = FALSE, display_progress = TRUE, ...) {
+                       overwrite = FALSE, display_progress = TRUE,
+                       add_tracking = TRUE, ...) {
   dots = list(...)
   # change locale
   locale_back = Sys.getlocale("LC_ALL")
@@ -89,15 +91,16 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
   extract_features = as.logical(extract_features); assert(extract_features, len = 1, alw = c(TRUE, FALSE))
   display_progress = as.logical(display_progress); assert(display_progress, len = 1, alw = c(TRUE, FALSE))
   verbose = as.logical(verbose); assert(verbose, len = 1, alw = c(TRUE, FALSE))
+  add_tracking = as.logical(add_tracking); assert(add_tracking, len = 1, alw = c(TRUE, FALSE))
   if(verbose) {
     verbosity = as.integer(verbosity); assert(verbosity, len = 1, alw = c(1,2))
     VER = ifelse(verbose & (verbosity==2), TRUE, FALSE)
   } else {
     VER = FALSE
   }
+  fileName = enc2native(normalizePath(fileName, winslash = "/", mustWork = FALSE))
   r_endian = cpp_checkTIFF(fileName)
   f_Ext = getFileExt(fileName)
-  fileName = normalizePath(fileName, winslash = "/", mustWork = FALSE)
   title_progress = basename(fileName)
   assert(write_to, len = 1, typ = "character")
   splitf_obj = splitf(fileName)
@@ -193,6 +196,7 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
   offsets = subsetOffsets(offsets = offsets, objects = objects, image_type = c("img", "msk"))
   offsets = c(IFDs[[1]]$curr_IFD_offset, offsets)
   off_number = length(offsets)
+  fname2 = charToRaw(fileName)
   
   # unwanted tags
   # 33004 corresponds to file date
@@ -352,33 +356,38 @@ subsetXIF <- function (fileName, write_to, objects, offsets, fast = TRUE,
                   # modify total objects count
                   buildIFD(val = L, typ = 4, tag = 33018, endianness = r_endian),
                   # add pkg version tag /!\ mandatory to prevent overwriting original file
-                  buildIFD(val = paste0(unlist(packageVersion("IFC")), collapse = "."), typ = 2, tag = 33090, endianness = r_endian),
-                  # add fileName tag /!\ allow to track where exported objects are coming from
-                  buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33091")),
-                                          fileName),
-                                        collapse = ">"),
-                           typ = 2, tag = 33091, endianness = r_endian),
-                  # add checksum tag /!\ allow to track where exported objects are coming from
-                  buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33092")),
-                                          checksumXIF(fileName)),
-                                        collapse = ">"), 
-                           typ = 2, tag = 33092, endianness = r_endian)
+                  buildIFD(val = paste0(unlist(packageVersion("IFC")), collapse = "."), typ = 2, tag = 33090, endianness = r_endian)
         )
+        if(add_tracking) {
+          extra = c(extra,
+                    # add fileName tag /!\ allow to track where exported objects are coming from
+                    buildIFD(val = collapse_raw(c(list(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33091", raw = TRUE))),
+                                                  list(fname2)),
+                                                collapse=as.raw(0x3e)),
+                             typ = 2, tag = 33091, endianness = r_endian),
+                    # add checksum tag /!\ allow to track where exported objects are coming from
+                    buildIFD(val = collapse_raw(c(list(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33092", raw = TRUE))),
+                                                  list(charToRaw(num_to_string(checksumXIF(fileName))))),
+                                                collapse=as.raw(0x3e)), 
+                             typ = 2, tag = 33092, endianness = r_endian)
+          )
+        }
       } else {
-        extra = c(extra, 
-                  # register current object id in new tag to be able to track it
-                  buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33093")),
-                                          OBJECT_ID),
-                                        collapse = ">"),
-                           typ = 2, tag = 33093, endianness = r_endian),
-                  
-                  # register current fil_ori in new tag to be able to track it
-                  buildIFD(val = paste0(c(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33094")),
-                                          fileName),
-                                        collapse = ">"), 
-                           typ = 2, tag = 33094, endianness = r_endian)
-        )
-        
+        if(add_tracking) {
+          extra = c(extra, 
+                    # register current object id in new tag to be able to track it
+                    buildIFD(val = collapse_raw(c(list(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33093", raw = TRUE))),
+                                                  list(charToRaw(num_to_string(OBJECT_ID)))),
+                                                collapse = as.raw(0x3e)),
+                             typ = 2, tag = 33093, endianness = r_endian),
+                    
+                    # register current fil_ori in new tag to be able to track it
+                    buildIFD(val = collapse_raw(c(list(suppressWarnings(getFullTag(IFD = structure(list(IFD), class = "IFC_ifd_list", "fileName_image" = fileName), which = 1, tag = "33094", raw = TRUE))),
+                                                  list(fname2)),
+                                                collapse=as.raw(0x3e)),
+                             typ = 2, tag = 33094, endianness = r_endian)
+          )
+        }
         # TODO if we remove mask tags it may be better to use offsets names as tracking object id
         # modify object id
         tmp = cpp_uint32_to_raw(floor(obj_id/XIF_step))
